<?php
// $Id: slot_machine.views.inc,v 1.1.2.1 2008/08/21 02:18:52 mcarbone Exp $

/**
 * @file slot_machine.views.inc
 * Provides support for the Views module.
 */

/**
 * Implementation of hook_views_tables()
 */
function slot_machine_views_tables() {
  $tables['slot_machine'] = array(
    'name' => 'slot_machine',
    'join' => array(
      'left' => array(
        'table' => 'node',
        'field' => 'nid'
      ),
      'right' => array(
        'field' => 'nid'
      ),
    ),
    'filters' => array(
      'topic' => array(
        'name' => t('Slot Machine: Topic'),
        'value' => array(
          '#type' => 'select',
          '#options' => 'slot_machine_handler_topic_list',
        ),
        'operator' => 'views_handler_operator_or',
        'help' => t('Filter the view to a specific Topic.'),
      ),
      'feature_type' => array(
        'name' => t('Slot Machine: Feature Type'),
        'value' => array(
          '#type' => 'select',
          '#options' => 'slot_machine_handler_feature_type_list',
        ),
        'operator' => 'views_handler_operator_or',
        'help' => t('Filter the view to a specific Feature Type. Without a "Slot Machine: Topic" filter, this could lead to duplicate nodes.'),
      ),
      'slot' => array(
        'name' => t('Slot Machine: Slot'),
        'operator' => array('IS' => t('Is')),
        'value' => array(
          '#type' => 'select',
          '#options' => 'slot_machine_handler_slot_list',
        ),
        'handler' => 'slot_machine_handler_filter_slot',
        'help' => t('Filter based on whether the node is live in the selected slot. Without a "Slot Machine: Topic" filter, this could lead to duplicate nodes.'),
      ),
      'current' => array(
        'name' => t('Slot Machine: Schedule Status'),
        'operator' => array('=' => t('Equals')),
        'value' => array(
          '#type' => 'select',
          '#options' => array(1 => t('Live'), 0 => t('Queued')),
        ),
        'handler' => 'slot_machine_handler_filter_live',
        'help' => t('Filter based on whether the node is live on the schedule or not. Without a "Slot Machine: Topic" filter, this could lead to duplicate nodes.'),
      ),
    ),
    'sorts' => array(
      'weight' => array(
        'name' => t('Slot Machine: Weight'),
        'help' => t('Sort by the slot weight -- only works with nodes that are currently live in the schedule.'),
        'handler' => 'slot_machine_sort_handler_weight',
      ),
      'priority' => array(
        'name' => t('Slot Machine: Priority'),
        'help' => t('Sort by priority -- mostly helpful when dealing with queued nodes.'),
      ),
     ),
  );
  return $tables;
}

/**
 * Generate a list of slot machine topics for use in handlers.
 */
function slot_machine_handler_topic_list($op, $argument) {
  $vocab = variable_get('slot_machine_topics', 'none');
  if (variable_get('slot_machine_home_topic', 1)) {
    $options = array(0 => 'Home');
  }
  else {
    $options = array();
  }
  $options += _sm_get_vocabulary_terms($vocab);
  return $options;
}

/**
 * Generate a list of feature types for use in handlers.
 */
function slot_machine_handler_feature_type_list($op, $argument) {
  $options = array();
  $content_types = _slot_machine_get_content_types();
  $feature_types = db_query('SELECT feature_type FROM {slot_machine_slots}');
  while ($obj = db_fetch_object($feature_types)) {
    $options[$obj->feature_type] = $content_types[$obj->feature_type];
  }
  return $options;
}

/**
 * Generate a list of slots for use in handlers.
 */
function slot_machine_handler_slot_list($op, $argument) {
  $options = array();
  $slots = db_query('SELECT sid, name FROM {slot_machine_slots}');
  while ($obj = db_fetch_object($slots)) {
    $options[$obj->sid] = $obj->name;
  }
  return $options;
}

function slot_machine_handler_filter_live($op, $filter, $filterinfo, &$query) {
  $value = $filter['value'];
  
  if (!$query->get_table_name('slot_machine', 1)) {
    // slot_machine table
    $joininfo = array(
      'type' => 'inner',
      'left' => array(
        'table' => 'node',
        'field' => 'nid',
      ),
      'right' => array(
        'field' => 'nid',
      ),
    );
    $query->add_table('slot_machine', FALSE, 1, $joininfo);
  }

  // slot_machine_slots table
  $joininfo = array(
    'left' => array(
      'table' => 'slot_machine',
      'field' => 'feature_type',
    ),
    'right' => array(
      'field' => 'feature_type AND {slot_machine}.priority = {slot_machine_slots}.priority',
    ),
  );
  $num = $query->add_table('slot_machine_slots', FALSE, 1, $joininfo);
  $tablename = $query->get_table_name('slot_machine_slots', $num);

  if (!$value) {
    $query->add_where("$tablename.feature_type IS NULL");
  }
  else {
    $query->add_where("$tablename.feature_type IS NOT NULL");  
  }
}

function slot_machine_handler_filter_slot($op, $filter, $filterinfo, &$query) {
  $value = $filter['value'];
  
  if (!$query->get_table_name('slot_machine', 1)) {
    // slot_machine table
    $joininfo = array(
      'type' => 'inner',
      'left' => array(
        'table' => 'node',
        'field' => 'nid',
      ),
      'right' => array(
        'field' => 'nid',
      ),
    );
    $query->add_table('slot_machine', FALSE, 1, $joininfo);
  }

  // slot_machine_slots table
  $joininfo = array(
    'type' => 'inner',
    'left' => array(
      'table' => 'slot_machine',
      'field' => 'feature_type',
    ),
    'right' => array(
      'field' => 'feature_type AND {slot_machine}.priority = {slot_machine_slots}.priority',
    ),
  );
  $num = $query->add_table('slot_machine_slots', FALSE, 1, $joininfo);
  $tablename = $query->get_table_name('slot_machine_slots', $num);

  $query->add_where("$tablename.sid = %d", $value);
}

function slot_machine_sort_handler_weight($action, &$query, $sortinfo, $sort) {
  if (!$query->get_table_name('slot_machine_slots', 1)) {
    // slot_machine_slots table
    $joininfo = array(
      'left' => array(
        'table' => 'slot_machine',
        'field' => 'feature_type',
      ),
      'right' => array(
        'field' => 'feature_type AND {slot_machine}.priority = {slot_machine_slots}.priority',
      ),
    );
    $num = $query->add_table('slot_machine_slots', FALSE, 1, $joininfo);
    $tablename = $query->get_table_name('slot_machine_slots', $num);
  }
  else {
    $tablename = $query->get_table_name('slot_machine_slots', 1);
  }

  $query->orderby[] = "$tablename.weight $sort[sortorder]";
}

/**
 * Implementation of hook_views_arguments()
 */
function slot_machine_views_arguments() {
  $arguments = array(
    'slot_machine_tid' => array(
      'name' => t('Slot Machine: Topic ID'),
      'handler' => 'slot_machine_handler_arg_topic',
      'help' => t('The Term ID argument allows users to filter a view by specifying a term ID (or 0 for "Home").'),
    ),
    'slot_machine_feature_type' => array(
      'name' => t('Slot Machine: Feature Type'),
      'handler' => 'slot_machine_handler_arg_feature_type',
      'help' => t('The feature type argument allows users to filter a view by specifying a feature type (e.g., "story").'),
    ),
  );
  return $arguments;
}

function slot_machine_handler_arg_topic($op, &$query, $argtype, $arg = '') {
  switch($op) {
    case 'summary':
      $query->ensure_table('slot_machine', true);
      $query->add_field('topic', 'slot_machine');
      $query->add_where('slot_machine.topic IS NOT NULL');
      $fieldinfo['field'] = "slot_machine.topic";
      return $fieldinfo;
      break;
    case 'sort':
      $query->add_orderby('slot_machine', 'topic', $argtype);
      break;
    case 'filter':
      $topic = intval($arg);
      $query->ensure_table('slot_machine', true);
      $query->add_where("slot_machine.topic = %d", $topic);
      break;
    case 'link':
      if ($query) {
        $name = db_result(db_query("SELECT name FROM {term_data} WHERE tid = %d", $query->topic));
      }
      elseif ($query == 0) {
        $name = t('Home');
      }
      return l($name, "$arg/" . intval($query->topic));
    case 'title':
      if ($query) {
        $name = db_result(db_query("SELECT name FROM {term_data} WHERE tid = %d", $query));
      }
      elseif ($query == 0) {
        $name = t('Home');
      }
      return $name;
  }
}

function slot_machine_handler_arg_feature_type($op, &$query, $argtype, $arg = '') {
  switch($op) {
    case 'summary':
      $query->ensure_table('slot_machine', true);
      $query->add_field('feature_type', 'slot_machine');
      $query->add_where('slot_machine.feature_type IS NOT NULL');
      $fieldinfo['field'] = "slot_machine.feature_type";
      return $fieldinfo;
      break;
    case 'sort':
      $query->add_orderby('slot_machine', 'feature_type', $argtype);
      break;
    case 'filter':
      $query->ensure_table('slot_machine', true);
      $query->add_where("slot_machine.feature_type = '%s'", $arg);
      break;
    case 'link':
      $title = node_get_types('name', $query);
      return l($title ? $title : check_plain($query), "$arg/" . intval($query->feature_type));
    case 'title':
      $title = node_get_types('name', $query);
      return $title ? $title : check_plain($query);
  }
}
